import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.awt.Shape;

/**
 * Zeichnet eine Ellipse auf die Leinwand.
 * 
 * @author Bernd Gramlich
 * @version 2015-10-06
 */
public class Ellipse
{
    // Attribute
    private int positionX;
    private int positionY;
    private int breite;
    private int hoehe;    
    private String farbe;
    private boolean sichtbar;

    /**
     * Konstruktor der Klasse Ellipse.
     * Erzeugt eine neue Ellipse in Standardgroesse und Standardfarbe.
     */
    public Ellipse ()
    {
        hoehe    = 20;
        breite   = 30;
        farbe    = "rot";
        sichtbar = false;
    }
    
	/**
	 * Mache diese Ellipse sichtbar.
	 */
	public void werdeSichtbar ()
	{
		sichtbar = true;
		zeichne ();
	}

	/**
	 * Mache diese Ellipse unsichtbar.
	 */
	public void werdeUnsichtbar ()
	{
		loesche ();
		sichtbar = false;
	}
	
    /**
	 * Aendere die Hoehe dieser Ellipse.
	 */
	public void setzeHoehe (int neueHoehe)
	{
		hoehe = neueHoehe;
		zeichne ();
	}
	
	/**
	 * Aendere die Breite dieser Ellipse.
	 */
	public void setzeBreite (int neueBreite)
	{
		breite = neueBreite;
		zeichne ();
	}
		
	/**
	 * Aendere die Farbe dieser Ellipse in 'neueFarbe'. Gueltige Angaben
	 * sind in der Methode setzeZeichenfarbe () der Klasse Leinwand
     * definiert. (Zuletzt waren das "blau", "gelb", "grau", "gruen", "lila",
     * "orange", "rot", "schwarz", "tuerkis" und "weiss".)
	 */
	public void setzeFarbe (String neueFarbe)
	{
		farbe = neueFarbe;
		zeichne ();
	}
		
	/**
	 * Nenne die Farbe dieser Ellipse.
	 */
	public String gibFarbe ()
	{
		return farbe;
	}
	
	/**
	 * Pruefe, ob ein Punkt in dieser Ellipse enthalten ist.
	 */
	public boolean enthaelt (Point2D punkt)
	{
		Shape form = Leinwand.gibLeinwand ().gibForm (this);
		
		if (null != form)
		{
		    return form.contains (punkt);
		}
		else
		{
		    return false;
		}
	}
	
	/**
	 * Aendere die Position dieser Ellipse.
	 */
	public void setzePosition (int neuePositionX, int neuePositionY)
	{
		positionX = neuePositionX;
		positionY = neuePositionY;
		zeichne ();
	}
	
	/**
	 * Verschiebe diese Ellipse waagrecht um 'entfernung'
	 * Bildschirmpunkte.
	 */
	public void verschiebeWaagrecht (int entfernung)
	{
		loesche ();
		positionX += entfernung;
		zeichne ();
	}

	/**
	 * Verschiebe diese Ellipse senkrecht um 'entfernung'
	 * Bildschirmpunkte.
	 */
	public void verschiebeSenkrecht (int entfernung)
	{
		loesche ();
		positionY += entfernung;
		zeichne ();
	}
		
	/**
	 * Zeichne diese Ellipse auf die Leinwand.
	 */
	private void zeichne ()
	{
		if (sichtbar)
		{
			Leinwand leinwand = Leinwand.gibLeinwand ();
			leinwand.zeichne (this, farbe, new Ellipse2D.Double (positionX,
					positionY, breite, hoehe));
		}
	}

	/**
	 * Loesche diese Ellipse von der Leinwand.
	 */
	private void loesche ()
	{
		if (sichtbar) {
			Leinwand leinwand = Leinwand.gibLeinwand ();
			leinwand.entferne (this);
		}
	}
}
