/**
 * Ein Knoten einer Schlange aus Piktogrammen.
 * 
 * @author Bernd Gramlich 
 * @version 2017-10-10
 */
public class Knoten
{
    /*
     * Inhalt des Knotens ist ein Piktogramm.
     */
    private Person inhalt;

    /*
     * Nachfolger des Knotens ist wieder ein Knoten.
     */
    private Knoten nachfolger;

    /**
     * Konstruktor.
     * Erzeuge gem den Parametern einen neuen Knoten ohne Nachfolger
     * und zeichne sein Piktogramm ab den Koordinaten (x|y).
     */
    public Knoten (int x, int y, boolean weiblich, String name,
    String hautfarbe, String haarfarbe, String schuhfarbe,
    String kleiderfarbe1, String kleiderfarbe2, String kleiderfarbe3)
    {
        nachfolger = null;

        if (weiblich)
        {
            inhalt = new Frau (3, x, y, name, hautfarbe, haarfarbe, schuhfarbe,
                kleiderfarbe1, kleiderfarbe2, kleiderfarbe3);
        }
        else
        {   
            inhalt = new Mann (3, x, y, name, hautfarbe, haarfarbe, schuhfarbe,
                kleiderfarbe1, kleiderfarbe2, kleiderfarbe3);
        }
    }

    /**
     * Nenne den Nachfolger des Knotens.
     */
    public Knoten gibNachfolger ()
    {
        return nachfolger;
    }

    /**
     * Nenne den Inhalt des Knotens (also die Person, die der Knoten enthlt).
     */
    public Person gibInhalt ()
    {
        return inhalt;
    }

    /**
     * Verschaffe dem Knoten einen neuen Nachfolger.
     */
    public void setzeNachfolger (Knoten neuerNachfolger)
    {
        nachfolger = neuerNachfolger;
    }

    /**
     * Verschiebe den Inhalt des Knotens und aller seiner Nachfolger um 100 Bildpunkte
     * nach links.
     */
    public void verschiebeNachLinks ()
    {
        /*
         * Ich verschiebe mein eigenes Bild nach links.
         */
        inhalt.verschiebe (-100, 0);

        /*
         * Wenn ich Nachfolger habe, sollen sie sich auch nach links verschieben.
         */
        if (null != nachfolger)
        {
            nachfolger.verschiebeNachLinks ();
        }
    }

    /**
     * Stelle hinten an die Schlange den neuen Kunden mit den angebenen
     * Daten an. Ich selbst bin der n-te Kunde.
     */
    public void stelleHintenAn (int n, boolean weiblich, String name,
    String hautfarbe, String haarfarbe, String schuhfarbe,
    String kleiderfarbe1, String kleiderfarbe2, String kleiderfarbe3)
    {
        if (null == nachfolger)
        {
            /*
             * Wenn ich keinen Nachfolger habe, wird der neue Kunde mein
             * neuer Nachfolger.
             */
            nachfolger = new Knoten (n*100, 100, weiblich, name, hautfarbe,
                haarfarbe, schuhfarbe, kleiderfarbe1, kleiderfarbe2,
                kleiderfarbe3);
        }
        else
        {
            /*
             * Wenn ich einen Nachfolger habe, berlasse ich ihm die Aufgabe,
             * den neuen Kunden hinten anzustellen. Mein Nachfolger ist dann
             * ist dabei der (n+1)-te Kunde.
             */
            nachfolger.stelleHintenAn (n+1, weiblich, name, hautfarbe,
                haarfarbe, schuhfarbe, kleiderfarbe1, kleiderfarbe2,
                kleiderfarbe3);
        }
    }
    
    /**
     * Lsche die erste Person hinter mir, die den angegebenen Namen trgt.
     */
    public Person loescheHinterMir (String name)
    {
        /*
         * Falls ich keinen Nachfolger habe ...
         */
        if (null == nachfolger)
        {
            /*
             * ... melde ich zurck, dass keine Person mit dem angegebenen
             * Namen hinter mir existiert.
             */
            return null;
        }
        /*
         * Falls aber mein Nachfolger den angegebenen Name trgt, ...
         */
        else if (nachfolger.gibInhalt ().gibName ().equals (name))
        {
            /*
             * ,,, mache ich ihn unsichtbar und lasse alle seine Nachfolger an
             * seinen Platz aufrcken.
             */
            Person rueckgabe = nachfolger.gibInhalt ();
            rueckgabe.werdeUnsichtbar ();
            nachfolger.gibNachfolger ().verschiebeNachLinks ();           
            
            /*
             * Dann lsche ich ihn, indem ich seinen Nachfolger zu meinem
             * neuen Nachfolger mache.
             */
            nachfolger = nachfolger.gibNachfolger ();
            
            /*
             * Den Inhalt meines ursprnglichen Nachfolgers gebe ich als
             * gesuchte Person zurck.
             */
            return rueckgabe;
            
        }
        /*
         * Andernfalls ...
         */
        {
            /*
             * ... frage ich meinen Nachfolger nach der ersten Person mit
             * dem angebenen Namen hinter ihm.
             */
            return nachfolger.loescheHinterMir (name);
        }
    }
    
    /**
     * Wie viele Knoten befinden sich ab mir in der Schlange?
     */
    public int gibLaengeAbMir ()
    {
        if (null == nachfolger)
        {
            /*
             * Wenn hinter mir niemand ist, ist die Schlange ab mir einen Knoten
             * lang.
             */
            return 1;
        }
        else
        {
            /*
             * Andernfalls ist die Schlange ab mir um 1 lnger als die Schlange ab
             * meinem Nachfolger.
             */
            return 1 + nachfolger.gibLaengeAbMir ();
        }
    }
    
    /**
     * Zhle alle Frauen von mir selbst ab.
     */
    public int gibFrauenzahl ()
    {
        int meineFrauenzahl;
        
        if (inhalt.istWeiblich ())
        {
            meineFrauenzahl = 1;
        }
        else
        {
            meineFrauenzahl = 0;
        }
        
        if (null == nachfolger)
        {
            return meineFrauenzahl;
        }
        else
        {
            return meineFrauenzahl + nachfolger.gibFrauenzahl ();
        }
    }
}